<?php

namespace Ems\Repository;

use Doctrine\ORM\EntityManager;
use Doctrine\DBAL\Connection;

class MovimentacaoRepository {
    /** @var EntityManager */
    protected $entityManager;

    /** @var Connection */
    protected $connection;

    public function __construct(
        EntityManager $entityManager
    ) {
        $this->entityManager = $entityManager;
        $this->connection = $entityManager->getConnection();
    }

    public function getUltimaMovimentacaoLegal($nrorg, $dtmescompetencia, $nrvinculom) {
        $sql = <<<SQL
   SELECT
       * 
    FROM
        GPE_MOVIMENTACAO MVT 
    WHERE
       MVT.NRORG = :P_NRORG 
       AND MVT.NRVINCULOM = :P_NRVINCULOM 
       AND MVT.NRTIPOESTRUTURA = (
            SELECT NRTPESTRUTLEGAL
            FROM GPE_PARAMORG
            WHERE NRORG = MVT.NRORG
       )
       AND MVT.DTINIMOVIMENT = 
       (
          SELECT
            MAX(DTINIMOVIMENT) 
          FROM
            GPE_MOVIMENTACAO 
          WHERE
            DTINIMOVIMENT <= LAST_DAY(:P_DTMESCOMPETENC) 
            AND NRORG = MVT.NRORG 
            AND NRVINCULOM = MVT.NRVINCULOM
            AND NRTIPOESTRUTURA = MVT.NRTIPOESTRUTURA
       )
SQL;

        
        $query = $this->entityManager->getConnection()->prepare($sql);

        $query->execute(
            array(
                'P_NRORG'           => $nrorg,
                'P_DTMESCOMPETENC'  => $dtmescompetencia,
                'P_NRVINCULOM'      => $nrvinculom
            )
        );

        $resultQuery = $query->fetchAll();


        if (isset($resultQuery)) {
            return $resultQuery;
        } else {
            return null;
        }
    }

    public function getUltimaMovimentacaoGeren($nrorg, $dtmescompetencia, $nrvinculom) {
        $sql = <<<SQL
   SELECT
       * 
    FROM
        GPE_MOVIMENTACAO MVT 
    WHERE
       MVT.NRORG = :P_NRORG 
       AND MVT.NRVINCULOM = :P_NRVINCULOM 
       AND MVT.NRTIPOESTRUTURA = (
            SELECT NRTPESTRUTGEREN
            FROM GPE_PARAMORG
            WHERE NRORG = MVT.NRORG
       )
       AND MVT.DTINIMOVIMENT = 
       (
          SELECT
            MAX(DTINIMOVIMENT) 
          FROM
            GPE_MOVIMENTACAO 
          WHERE
            DTINIMOVIMENT <= LAST_DAY(:P_DTMESCOMPETENC) 
            AND NRORG = MVT.NRORG 
            AND NRVINCULOM = MVT.NRVINCULOM
            AND NRTIPOESTRUTURA = MVT.NRTIPOESTRUTURA
       )
SQL;

        
        $query = $this->entityManager->getConnection()->prepare($sql);

        $query->execute(
            array(
                'P_NRORG'           => $nrorg,
                'P_DTMESCOMPETENC'  => $dtmescompetencia,
                'P_NRVINCULOM'      => $nrvinculom
            )
        );

        $resultQuery = $query->fetchAll();


        if (isset($resultQuery)) {
            return $resultQuery;
        } else {
            return null;
        }
    }

    public function setMovimentacao(
        array $params
    ): void {
        $sql = <<< SQL
            INSERT INTO GPE_MOVIMENTACAO (
                NRMOVIMENTACAO,     NRORG,              NRVINCULOM,
                NRTIPOTRANSFER,     NRTPMOVTRANSFM,     NRMOTIVOTRANSF,
                NRESTRUTURAM,       DTINIMOVIMENT,      DTFIMMOVIMENT,
                DSOBSERVACAO,       DTINCLUSAO,         DTULTATU,
                NRORGINCLUSAO,      CDOPERINCLUSAO,     NRORGULTATU,
                CDOPERULTATU,       NRTIPOESTRUTURA,    DTMOVIMENTRETRO,
                IDATUALIZAACERTO,   NRMOTIVOAFASTA
            )
            VALUES (
                :NRMOVIMENTACAO,    :NRORG,             :NRVINCULOM,
                :NRTIPOTRANSFER,    :NRTPMOVTRANSFM,    :NRMOTIVOTRANSF,
                :NRESTRUTURAM,      :DTINIMOVIMENT,     :DTFIMMOVIMENT,
                :DSOBSERVACAO,      :DTINCLUSAO,        :DTULTATU,
                :NRORGINCLUSAO,     :CDOPERINCLUSAO,    :NRORGULTATU,
                :CDOPERULTATU,      :NRTIPOESTRUTURA,   :DTMOVIMENTRETRO,
                :IDATUALIZAACERTO,  :NRMOTIVOAFASTA
            )
SQL;

        $std = $this->connection->prepare($sql);
        $std->execute($params);
    }
}